# Created by Q-ays.
# whosqays@gmail.com

# install elasticsearch_dsl before use

# elastic-search tools

from elasticsearch_dsl import Q


def o2d(obj):
    is_arr = isinstance(obj, list)
    is_set = isinstance(obj, set)

    if is_set or is_arr:
        res = []
        for o in obj:
            if not isinstance(o, dict):
                d0 = o.to_dict(include_meta=True)
                d1 = d0.get('_source')
                d1['id'] = d0.get('_id')
                res.append(d1)
            else:
                if o.get('_source'):
                    d0 = o.get('_source')
                    d0['id'] = o.get('_id')
                    res.append(d0)
                else:
                    res.append(o)

        return res

    if not isinstance(obj, dict):
        d0 = obj.to_dict(include_meta=True)
        d1 = d0.get('_source')
        d1['id'] = d0.get('_id')
        return d1
    else:
        if obj.get('data'):
            data = obj.get('data')
            obj['data'] = o2d(data)

        return obj


doc_type0 = 'doc'


def repo_ref(doc_type):
    """
    基于es数据库增删改查的公共类
    :param doc_type:
    :return:
    """
    class BaseRepo:

        def __init__(self, ModelType, doc_type=doc_type):
            self.Model = ModelType
            self.index = ModelType.Index.name
            self.type = doc_type

        def add(self, data):
            print(data)
            model = self.Model(**data)
            model.save()
            return model

        def delete(self, did=None):
            if did:
                res = self.Model.get(did)
                res.delete()
                return res

        def update(self, did, data):
            model = self.Model.get(did)
            for key in data:
                if hasattr(model, key):
                    setattr(model, key, data[key])
            model.save()
            return model

        def get(self, did=None, p=None, **kwargs):
            """
            查询对象字段时，参数为对象
            :param did:
            :param p:
            :param kwargs:
            :return:
            """
            if did:
                if isinstance(did, list):
                    obj = self.Model.mget(did)
                else:
                    obj = self.Model.get(did)
            else:
                s = self.Model.search()
                for key in kwargs:
                    if isinstance(kwargs[key], dict):
                        s = s.query('nested', path=key, query=Q('match', **{str(key) + '.id': kwargs[key].get('id')}))
                    else:
                        s = s.query('match', **{key: kwargs[key]})

                if isinstance(p, list) or isinstance(p, tuple):
                    s = s[p[0]:p[1]]
                    res = s.execute()
                else:
                    res = s.scan()

                obj = []
                for o in res:
                    obj.append(o)

            return obj

        def paging(self, p=(0, 10), **kwargs):
            s = self.Model.search()
            for key in kwargs:
                if isinstance(kwargs[key], dict):
                    s = s.query('nested', path=key, query=Q('match', **{str(key) + '.id': kwargs[key].get('id')}))
                else:
                    s = s.query('match', **{key: kwargs[key]})

            if isinstance(p, list) or isinstance(p, tuple):
                s = s[p[0]:p[1]]
                res = s.execute()
                obj = []
                for o in res:
                    obj.append(o)

                data = dict()
                data['loc'] = p[0]
                data['size'] = p[1]
                data['total'] = res.hits.total
                data['data'] = obj

                return data

    return BaseRepo


Repo = repo_ref(doc_type0)



# Created by Q-ays.
# whosqays@gmail.com


from enum import Enum, unique, IntEnum


@unique
class Code(Enum):
    ERROR = -1, '错误'
    MS_ERROR = -2, '微服务错误'
    DATABASE_ERROR = -3, '数据库错误'
    TIMEOUT = -4, '超时'
    USER_ERROR = -5, '用户出错',
    TOKEN_ERROR = -6, 'token验证出错'

    SUCCESS = 1, '成功'
    WARNING = 99, '警告'
    UNKNOWN = 0, '未知'

    def __init__(self, code, msg):
        self._code = code
        self.msg = msg

    @property
    def get_code(self):
        return self._code

    @property
    def get_msg(self):
        return self.msg


class WisdomErrorException(Exception):
    """
    顶层异常类
    """

    def __init__(self, desc=None, code=0, code_enum=Code.UNKNOWN):
        self.codeEnum = code_enum
        self.code = code
        self.desc = desc

    def gene_code(self):
        if self.__class__.__name__ == 'WisdomErrorException':
            return self.assemble_error_code(0, self.my_code)
        parent = self.__class__.mro()[1]()
        return self.assemble_error_code(parent.my_code, self.my_code)

    @staticmethod
    def assemble_error_code(parent: int, child: int):
        return -(10000 + abs(parent) * 100 + abs(child))

    @property
    def my_desc(self):
        if hasattr(self, 'desc'):
            return self.desc if self.desc else (
                '无描述信息-none described msg' if not hasattr(self, 'codeEnum') else self.codeEnum.get_msg)

    @property
    def my_msg(self):
        return '无内容-null of contents' if not hasattr(self, 'codeEnum') else self.codeEnum.get_msg

    @property
    def my_data(self):
        return None if not hasattr(self, 'data') else self.data

    @property
    def my_code(self):
        if hasattr(self, 'codeEnum'):
            return self.codeEnum.get_code if self.codeEnum else 0
        if hasattr(self, 'code'):
            return self.code if self.code else 0
        return 0

    def revert(self):
        return {"code": self.gene_code(), 'desc': self.my_desc, 'msg': self.my_msg, 'data': self.my_data}


class CommonException(WisdomErrorException):
    """
    公用报错异常类
    """

    def __init__(self, desc=None, code=-1, data=None, code_enum=Code.ERROR):
        self.desc = desc
        self.code = code
        self.data = data
        self.codeEnum = code_enum


class DbException(WisdomErrorException):
    """
    数据库异常父类
    """

    def __init__(self, desc=None, data=None, code_enum=Code.DATABASE_ERROR):
        self.data = data
        self.desc = desc
        self.codeEnum = code_enum


class UserException(WisdomErrorException):
    """
    用户操作异常父类
    """

    def __init__(self, desc=None, data=None, code_enum=Code.USER_ERROR):
        self.data = data
        self.desc = desc
        self.codeEnum = code_enum


class TokenException(UserException):
    """
    token 验证异常
    """

    def __init__(self, desc=None, data=None, code_enum=Code.TOKEN_ERROR):
        self.data = data
        self.desc = desc
        self.codeEnum = code_enum


class MsException(WisdomErrorException):
    """
    微服务异常父类
    """

    def __init__(self, desc=None, data=None, code_enum=Code.MS_ERROR):
        self.data = data
        self.desc = desc
        self.codeEnum = code_enum


class TimeoutException(WisdomErrorException):
    """
    超时异常父类
    """

    def __init__(self, desc=None, data=None, code_enum=Code.TIMEOUT):
        self.data = data
        self.desc = desc
        self.codeEnum = code_enum


class PostgresException(DbException):
    def __init__(self, desc='pg数据库错误', code=1):
        self.desc = desc
        self.code = code


class EsException(DbException):
    pass


if __name__ == '__main__':
    # CommonException('djfidso').gene_code()

    try:
        # raise WisdomErrorException()
        raise UserException()

    except WisdomErrorException as e:
        print(e.revert())



from wisdoms.exec import UserException


class InnerAuth:

    @staticmethod
    def all_true(*args):
        """
        所有为真通过
        :param args:
        :return:
        """
        for a in args:
            if a:
                pass
            else:
                raise UserException('权限验证失败')

    @staticmethod
    def one_true(*args):
        """
        验证至少一个为真则通过
        :param args:
        :return:
        """
        for a in args:
            if a:
                return

        raise UserException('权限验证失败')

    @staticmethod
    def mget(origin, key, exc='用户权限获取异常'):
        value = origin.get(key)

        if value is not None:
            return value
        else:
            raise UserException(exc)

    def permit_get(self, origin):
        return self.mget(origin, 'permit', '代理权限异常-agent exception')

    def privilege_get(self, origin):
        permit = self.permit_get(origin)
        return self.mget(permit, 'privilege', '获取用户特殊权限失败-privilege exception')

    def level_get(self, origin):
        privilege = self.privilege_get(origin)
        return self.mget(privilege, 'level', '获取用户特殊权限等级失败-privilege level exception')

    def role_get(self, origin):
        privilege = self.privilege_get(origin)
        return self.mget(privilege, 'role', '获取用户特殊权限角色失败-privilege role exception')

    def role_judge(self, origin, role0):
        role = self.role_get(origin)

        return role == role0

    def bloc(self, origin):
        """
        判断当前是否是集团代理用户
        :param origin:
        :return: boolean
        """
        return self.role_judge(origin, 'bloc')

    def agent(self, origin):
        """
        判断当前是否是区域代理用户
        :param origin:
        :return: boolean
        """
        return self.role_judge(origin, 'agent')

    def admin(self, origin):
        """
        判断是否是平台管理员用户
        :param origin:
        :return: boolean
        """
        return self.role_judge(origin, 'admin')

    def super(self, origin):
        """
        判断是否是超级管理用户
        :param origin:
        :return: boolean
        """
        return self.role_judge(origin, 'super')

    def region_in(self, origin, region0):
        """
        检查区域id是否在代理范围内
        :param origin:
        :param region0:
        :return: boolean
        """
        permit = self.permit_get(origin)
        region = self.mget(permit, 'region', '代理区域获取异常-region get exception')

        return region0 in region

    def agent_in(self, origin, agent0):
        """
        检查子代理是否在代理范围内
        :param origin:
        :param agent0:
        :return: boolean
        """
        permit = self.permit_get(origin)
        agent = self.mget(permit, 'agent', '子代理用户获取异常-sub agent get exception')

        return agent0 in agent

    def org_in(self, origin, org0):
        """
        检查组织id是否在代理组织列表内
        :param origin:
        :param org0:
        :return: boolean
        """
        permit = self.permit_get(origin)
        org = self.mget(permit, 'org', '代理组织获取异常-org get exception')

        return org0 in org


auth = InnerAuth()
